#!/bin/bash

# encoder: {{ data.network1 }}, decoder: {{ data.decoder }}, port: {{ data.pull_port }}

/usr/local/bin/pulltest \
    /p 1999 \
    /server {{ data.ip }} {{ data.pull_port }} \
    /rep '{{ data.reporter }}' \
    /s_login '{{ data.pulltest_login }}' \
    /s_pass '{{ data.pulltest_password }}' \
    /net1 '{{ data.decoder }}' \
    /pulltest 999999
