apiVersion: apps/v1
kind: Deployment
metadata:
  name: sendstream-{{ encoder_decoder_id }}-deployment
  namespace: pulltest
  labels:
    app: sendstream
spec:
  replicas: 1
  selector:
    matchLabels:
      app: sendstream
  template:
    metadata:
      labels:
        app: sendstream
    spec:
      imagePullSecrets:
      - name: regcred
      containers:
      - name: sendstream
        image: 193048895737.dkr.ecr.us-west-1.amazonaws.com/taylorm/termite_recharge/sendstream@sha256:b2c0e85c09e4e9c68603c1e68e71727177936d3aa57a9268e0b3a2e69228b7a5
        imagePullPolicy: Always
        env:
        - name: DRM
          value: {{ data.drm }}
        command:
         - /bin/bash
         - "-c"
         - |
           /bin/bash <<'EOF'
           # encoder: {{ data.network1 }}, decoder: {{ data.decoder }}, port: {{ data.pull_port }}
           /usr/local/bin/sendstream \
               -cleanup -loop \
               -port {{ data.pull_port }} \
               -ip {{ data.ip }} \
               -file /video/vaqua-6M-DYNO.actl3 \
               -network1 '{{ data.network1 }}' \
               -reporter '{{ data.reporter }}' \
               -slug '{{ data.slug }}'
           EOF
