apiVersion: apps/v1
kind: Deployment
metadata:
  name: pulltest-{{ encoder_decoder_id }}-deployment
  namespace: pulltest
  labels:
    app: pulltest
spec:
  replicas: 1
  selector:
    matchLabels:
      app: pulltest
  template:
    metadata:
      labels:
        app: pulltest
    spec:
      imagePullSecrets:
      - name: regcred
      containers:
      - name: pulltest
        image: 193048895737.dkr.ecr.us-west-1.amazonaws.com/taylorm/termite_recharge/pulltest@sha256:ef924fba6a6176c32864dbf56fa01f6ee0ae2ba0aca6fb0c6bdab9715da770b3
        imagePullPolicy: Always
        command:
         - /bin/bash
         - "-c"
         - |
           sleep 1999
