#!/bin/bash

# encoder: {{ data.network1 }}, decoder: {{ data.decoder }}, port: {{ data.pull_port }}

/usr/local/bin/sendstream \
    -cleanup -loop -reporter '{{ data.reporter }}' \
    -port {{ data.pull_port }} \
    -ip {{ data.ip }} \
    -file /video/vaqua-6M-DYNO.actl3 \
    -network1 '{{ data.network1 }}' \
    -slug '{{ data.slug }}'
